__doc__=f"""mrldb by Rémi "Mr e-RL" LANGDORPH
Copyright (c) 2019 Rémi LANGDORPH - mrerl@warlegend.net
under MIT License (https://github.com/merlleu/mrldb/blob/master/LICENSE)"""
from .struct import get_struct_init
import threading
from queue import Queue
class MrlDBSqlite:
    def __init__(self, file, structure=None, autocommit=0, debug=False):
        self.structure=structure
        self._config={"system":"sqlite", "file": file, "structure": f"{len(structure) if structure!=None else '0'} tables"}
        self.db=sqlite_dbthread(file, debug)
        self.cursor=self.db
        self.autocommit=None
        if autocommit!=0: self.autocommit=sqlite_autocommit(self.db, autocommit)
        return
    def insert(self, table, data):
        def frmt(d):
            return d.__repr__() if d is not None else "null"
        return self.db.execute(f"INSERT INTO {table}({', '.join([x for x in data.keys()])}) VALUES ({', '.join([frmt(x) for x in data.values()])})")
    def update(self, table, data, conds=None):
        def frmt(d):
            return d.__repr__() if d is not None else "null"
        return self.db.execute(f"UPDATE {table} SET {', '.join([key+'='+frmt(arg) for key, arg in data.items()])}{' WHERE '+conds if conds!=None else ''}")
    def select(self, table, columns, conds=None):
        if self.structure!=None:
            if columns=="*":columns=self.structure[table].keys()

            return [
            {_col:_var for _col, _var in zip(columns, record)}
            for record in
            self.db.select(f"SELECT {'*' if columns=='*' else ', '.join(columns)} FROM {table}{' WHERE '+conds if conds!=None else ''}")
            ]
        else:
            return self.db.select(f"SELECT {'*' if columns=='*' else ', '.join(columns)} FROM {table}{' WHERE '+conds if conds!=None else ''}")
    def delete(self, table, conds=None):
        # table= "mytable"
        # conds= "(x=1) or (x=2 and y=3)"
        return self.db.execute(f"DELETE FROM {table}{' WHERE '+conds if conds!=None else ''}")
    def _getinfos(self):
        return self._config
    def init(self):
        [self.db.execute(command) for command in get_struct_init(self.structure)]
        return self
    def __str__(self):
        return f"<mrldb.MrlDBSqlite at {id(self)} - connection: {self._config['file']} - autocommit: {f'{self.autocommit.timing} seconds' if self.autocommit!=None else 'disabled'}>"
    def __repr__(self):
        return f"<mrldb.MrlDBSqlite at {id(self)} - connection: {self._config['file']} - autocommit: {f'{self.autocommit.timing} seconds' if self.autocommit!=None else 'disabled'}>"

class sqlite_dbthread(threading.Thread):
    def __init__(self, db, debug):
        threading.Thread.__init__(self)
        self.db=db
        self.status=True
        self.debug=debug
        self.reqs=Queue()
        self.start()
    def run(self):
        import sqlite3
        cnx = sqlite3.Connection(self.db)
        cursor = cnx.cursor()
        while self.status:
            req, arg, res = self.reqs.get()
            if req=='--close--': break
            elif req=='--commit--': cnx.commit()
            try:
                if self.debug: print(self.db ,"\t",req, arg)
                cursor.execute(req, arg)
                if res:
                    for rec in cursor:
                        res.put(rec)
                    res.put('--no more--')
            except:res.put('--no more--')
        cnx.close()
    def execute(self, req, arg=None, res=None):
        self.reqs.put((req, arg or tuple(), res))
    def select(self, req, arg=None):
        res=Queue()
        self.execute(req, arg, res)
        while True:
            rec=res.get()
            if rec=='--no more--': break
            yield rec
    def commit(self):
    	self.execute("--commit--")
    def close(self):
        self.execute('--close--')
    def stop(self):
        self.status=False
class sqlite_autocommit(threading.Thread):
    def __init__(self, sql, t):
        threading.Thread.__init__(self)
        self._stopevent = threading.Event( )
        self.sql=sql
        self.timing=t
        self.start()
    def run(self):
        lastnbr=0
        idn=0
        while not self._stopevent.isSet():
            self._stopevent.wait(self.timing)
            self.sql.commit()
    def stop(self):
        self._stopevent.set( )



__doc__=f"""mrldb by Rémi "Mr e-RL" LANGDORPH
Copyright (c) 2019 Rémi LANGDORPH - mrerl@warlegend.net
under MIT License (https://github.com/merlleu/mrldb/blob/master/LICENSE)"""

from .struct import get_struct_init
import asyncio
try:
    from mysql.connector.errors import OperationalError
except:
    pass
class MrlDBMsqlAsync:
    def __init__(self, host, database=None, structure=None, user=None, password=None, autocommit=True):
        self.structure=structure
        self._config={"system":"mysql", "host": host, "database": database, "structure": f"{len(structure) if structure!=None else '0'} tables", "user": user, "password":password, "autocommit": autocommit}
        return
    @asyncio.coroutine
    def connect(self):
        import mysql_async.connector as mariadb
        self.connection = mariadb.Connect(host=self._config["host"], database=self._config["database"], user=self._config["user"], password=self._config["password"])
        self.connection.autocommit=self._config["autocommit"]
        yield from self.connection.connect()
        self.cursor = yield from self.connection.cursor()

    @asyncio.coroutine
    def restart(self):
        yield from self.cursor.close()
        yield from self.connection.close()
        import mysql_async.connector as mariadb
        self.connection = mariadb.Connect(host=self._config["host"], database=self._config["database"], user=self._config["user"], password=self._config["password"])
        self.connection.autocommit=self._config["autocommit"]
        yield from self.connection.connect()
        self.cursor = yield from self.connection.cursor()

    @asyncio.coroutine
    def insert(self, table, data):
        def frmt(d):
            return d.__repr__() if d is not None else "null"
        try:
            return (yield from self.cursor.execute(f"INSERT INTO {table}({', '.join([x for x in data.keys()])}) VALUES ({', '.join([frmt(x) for x in data.values()])})"))
        except OperationalError:
            yield from self.restart()
            return (yield from self.cursor.execute(f"INSERT INTO {table}({', '.join([x for x in data.keys()])}) VALUES ({', '.join([frmt(x) for x in data.values()])})"))
    @asyncio.coroutine
    def update(self, table, data, conds=None):
        def frmt(d):
            return d.__repr__() if d is not None else "null"
        try:
            return (yield from self.cursor.execute(f"UPDATE {table} SET {', '.join([key+'='+frmt(arg) for key, arg in data.items()])}{' WHERE '+conds if conds!=None else ''}"))
        except OperationalError:
            yield from self.restart()
            return (yield from self.cursor.execute(f"UPDATE {table} SET {', '.join([key+'='+frmt(arg) for key, arg in data.items()])}{' WHERE '+conds if conds!=None else ''}"))
    @asyncio.coroutine
    def select(self, table, columns, conds=None, nowrap=False):
        if columns=="*":columns=self.structure[table].keys()
        try:
            yield from self.cursor.execute(f"SELECT {'*' if columns=='*' else ', '.join(columns)} FROM {table}{' WHERE '+conds if conds!=None else ''}")
        except OperationalError:
            yield from self.restart()
            yield from self.cursor.execute(f"SELECT {'*' if columns=='*' else ', '.join(columns)} FROM {table}{' WHERE '+conds if conds!=None else ''}")
        if not nowrap:
            return [
            {_col:_var for _col, _var in zip(columns, record)}
            for record in
            (yield from self.cursor.fetchall())
            ]
        else:
            return (yield from self.cursor.fetchall())
    @asyncio.coroutine
    def delete(self, table, conds=None):
        try:
            return (yield from self.cursor.execute(f"DELETE FROM {table}{' WHERE '+conds if conds!=None else ''}"))
        except OperationalError:
            yield from self.restart()
            return (yield from cursor.execute(f"DELETE FROM {table}{' WHERE '+conds if conds!=None else ''}"))
    def _getinfos(self):
        return self._config
    async def init(self):
        [self.cursor.execute(command) for command in get_struct_init(self.structure)]
        return self
    def __str__(self):
        return f"<mrldb.MrlDBMsql at {id(self)} - connection: {self._config['host']}>"
    def __repr__(self):
        return f"<mrldb.MrlDBMsql at {id(self)} - connection: {self._config['host']}>"



__doc__=f"""mrldb by Rémi "Mr e-RL" LANGDORPH
Copyright (c) 2019 Rémi LANGDORPH - mrerl@warlegend.net
under MIT License (https://github.com/merlleu/mrldb/blob/master/LICENSE)"""
__all__=["MrlDBMsql", "MrlDBCassandra", "MrlDBSqlite", "MrlDBMsqlAsync"]
from .type_mysql import MrlDBMsql
from .type_cassandra import MrlDBCassandra
from .type_sqlite import MrlDBSqlite
from .type_aiomysql import MrlDBMsqlAsync



__doc__=f"""mrldb by Rémi "Mr e-RL" LANGDORPH
Copyright (c) 2019 Rémi LANGDORPH - mrerl@warlegend.net
under MIT License (https://github.com/merlleu/mrldb/blob/master/LICENSE)"""
def get_struct_init(structure):
    return [
    "CREATE TABLE IF NOT EXISTS "+table+"("+", ".join(
    [
    colname+" "+content for colname, content in cols.items()
    ])+")"
    for table, cols in structure.items()
    ]



__all__=["MrlDBMsql", "MrlDBCassandra", "MrlDBSqlite", "MrlDBCluster", "mdbcl", "mdbstr", "MrlDBMsqlAsync"]
from .db_types import *
from .mrl_cluster import *
from .dbutils import dbutils as mdbstr
##version#start##
__version__='0.1.2'
##version#end##
__doc__=f"""mrldb v{__version__} by Rémi "Mr e-RL" LANGDORPH
Copyright (c) 2019 Rémi LANGDORPH - mrerl@warlegend.net
under MIT License (https://github.com/merlleu/mrldb/blob/master/LICENSE)
"""



__doc__=f"""mrldb by Rémi "Mr e-RL" LANGDORPH
Copyright (c) 2019 Rémi LANGDORPH - mrerl@warlegend.net
under MIT License (https://github.com/merlleu/mrldb/blob/master/LICENSE)"""
__all__=["dbutils"]
class _dbutils:
    def __init__(self):
        pass
    def insert(self, table, data):
        # table= "mytable"
        # data= {"name": "tom", "age": 1}
        def frmt(d):
            return d.__repr__() if d is not None else "null"
        return f"INSERT INTO {table}({', '.join([x for x in data.keys()])}) VALUES ({', '.join([frmt(x) for x in data.values()])})"
    def update(self, table, data, conds=None):
        # table= "mytable"
        # data= {"name": "tom", "age": 1}
        # conds= "(x=1) or (x=2 and y=3)"
        def frmt(d):
            return d.__repr__() if d is not None else "null"
        return f"UPDATE {table} SET {', '.join([key+'='+frmt(arg) for key, arg in data.items()])}{' WHERE '+conds if conds!=None else ''}"
    def select(self, table, columns, conds=None):
        # table= "mytable"
        # columns= ["test", "x"]
        # conds= "(x=1) or (x=2 and y=3)"
        return f"SELECT {'*' if columns=='*' else ', '.join(columns)} FROM {table}{' WHERE '+conds if conds!=None else ''}"
    def delete(self, table, conds=None):
        # table= "mytable"
        # conds= "(x=1) or (x=2 and y=3)"
        return f"DELETE FROM {table}{' WHERE '+conds if conds!=None else ''}"
dbutils=_dbutils()



__doc__=f"""mrldb by Rémi "Mr e-RL" LANGDORPH
Copyright (c) 2019 Rémi LANGDORPH - mrerl@warlegend.net
under MIT License (https://github.com/merlleu/mrldb/blob/master/LICENSE)"""
from .struct import get_struct_init

class MrlDBCassandra:
    def __init__(self, cluster, db=None, structure=None, username=None, password=None):
        from cassandra.cqlengine import connection
        from cassandra.cluster import Cluster
        if isinstance(cluster, str):
            cluster=[cluster]
        if (username==None or password==None):
            self.cluster=Cluster(cluster)
        else:
            from cassandra.auth import PlainTextAuthProvider
            self.auth_provider = PlainTextAuthProvider(username=username, password=password)
            self.cluster_=Cluster(cluster, auth_provider=self.auth_provider)
        if db is not None:
            self.db=self.cluster.connect(db)
            self.cursor=self.db
        else:
            self.cluster_=self.cluster
            self.cursor=self.cluster_.connect()
        self.structure=structure
        self._config={"system":"cassandra", "cluster": cluster,"database": db, "structure": f"{len(structure) if structure!=None else '0'} tables", "username": username, "password":password}
        return
    def insert(self, table, data):
        def frmt(d):
            return d.__repr__() if d is not None else "null"
        return self.cursor.execute(f"INSERT INTO {table}({', '.join([x for x in data.keys()])}) VALUES ({', '.join([frmt(x) for x in data.values()])})")
    def update(self, table, data, conds=None):
        def frmt(d):
            return d.__repr__() if d is not None else "null"
        return self.cursor.execute(f"UPDATE {table} SET {', '.join([key+'='+frmt(arg) for key, arg in data.items()])}{' WHERE '+conds if conds!=None else ''}")
    def select(self, table, columns, conds=None):
        if self.structure!=None:
            if columns=="*":columns=self.structure[table].keys()
            return [
            {_col:_var for _col, _var in zip(columns, record)}
            for record in
            self.cursor.execute(f"SELECT {'*' if columns=='*' else ', '.join(columns)} FROM {table}{' WHERE '+conds if conds!=None else ''}")
            ]
        else:
            return self.cursor.execute(f"SELECT {'*' if columns=='*' else ', '.join(columns)} FROM {table}{' WHERE '+conds if conds!=None else ''}")
    def delete(self, table, conds=None):
        # table= "mytable"
        # conds= "(x=1) or (x=2 and y=3)"
        return self.cursor.execute(f"DELETE FROM {table}{' WHERE '+conds if conds!=None else ''}")
    def _getinfos(self):
        return self._config
    def init(self):
        [self.cursor.execute(command) for command in get_struct_init(self.structure)]
        return self
    def __str__(self):
        return f"<mrldb.MrlDBCassandra at {id(self)} - connection: {self._config['cluster']}>"
    def __repr__(self):
        return f"<mrldb.MrlDBCassandra at {id(self)} - connection: {self._config['cluster']}>"



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()
##version#start##
__version__='0.1.5'
##version#end##
setuptools.setup(
    name="mrldb",
    version=__version__,
    author="Rémi 'Mr e-RL'LANGDORPH",
    author_email="r.langdorph@gmail.com",
    description="A library to handle your databases: cassandra, mariadb, (mysql), sqlite3",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/merlleu/mrldb",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



__doc__=f"""mrldb by Rémi "Mr e-RL" LANGDORPH
Copyright (c) 2019 Rémi LANGDORPH - mrerl@warlegend.net
under MIT License (https://github.com/merlleu/mrldb/blob/master/LICENSE)"""
__all__=["MrlDBCluster", "mdbcl"]

_cluster={"main": None}
class MrlDBCluster:
    def __init__(self):
        self.dbs={}
        self.aliases={}
        _cluster["main"]=self
    def get(self, name):
        return self.dbs[self.aliases[name]]
    def add(self, name, db, aliases=[]):
        self.dbs[name]=db
        self.aliases[name]=name
        for alias in aliases: self.aliases[alias]=name
        return name
    def addalias(self, name, aliases):
        if isinstance(aliases, str):
            self.aliases[aliases]=name
        else:
            for alias in aliases: self.aliases[alias]=name
        return name
    def get_cluster_infos(self):
        return {
        name: db._getinfos() for name, db in self.dbs.items()
        }
    def __iter__(self):
        return self.dbs.items().__iter__()
    def __getitem__(self, key):
        return self.dbs[self.aliases[key]]
    def __str__(self):
        return f"<mrldb.MrlDBCluster at {id(self)} - {len(self.dbs)} connection(s)>"
    def __repr__(self):
        return f"<mrldb.MrlDBCluster at {id(self)} - {len(self.dbs)} connection(s)>"
mdbcl=None
try:
    from werkzeug.local import LocalProxy
    mdbcl=LocalProxy(lambda: _cluster["main"])
except: pass
